#include	<corlib.h>
#include	<mm_alloc.h>
#include	<string.h>

extern unsigned short	palette[];
extern unsigned char	pat[], vpat[];
extern unsigned char	map[];

extern void	SetPalet();

void
savemap( fname )
unsigned char	*fname;
{
	unsigned char	buf[32], *rptr, *prev;
	int	i, j;
	FILE	*fp;

	if( !fname )	return;
	fp= fopen( fname, "wb" );
	if( !fp ){
		ConsoleOpen();
		ConsolePrint( "file cannot write\n" );
		return;
	}

	/* buffer clear */
	for( i= 0; i < 32; i++ )	buf[i]= 0;

	/* palette save ( word / color * 16 ) */
	fwrite( palette, sizeof( unsigned short ), 16, fp );

	/* pattern save ( word / pixel * 256 ) */
	rptr= pat;
	for( i= 0; i < 32 / 16 * 256; i++ ){
		for( j= 0; j < 16; j++ ){
			buf[j * 2 + 1]= *rptr++;
		}
		fwrite( buf, sizeof( unsigned char ), 32, fp );
	}

	/* map save ( byte / unit * 24 * 1024 ) */
	rptr= map + 24 * ( 1024 - 1 );
	prev= rptr;
	for( j= 8; j < 32; j++ )	buf[j]= *rptr++;
	fwrite( buf, sizeof( unsigned char ), 32, fp );

	for( i= 0; i < 1024 - 1; i++ ){
		unsigned int flag= 0;
		rptr-= 48;
		for( j= 8; j < 32; j++ ){
			flag*= 2;
			buf[j]= *rptr;
			if( *rptr++ != *prev++ )	flag+= 1;
		}
		prev-= 48;
		buf[0]= ( flag >> 16 ) & 0xff;
		buf[1]= ( flag >> 8 ) & 0xff;
		buf[2]= flag & 0xff;
		fwrite( buf, sizeof( unsigned char ), 32, fp );
	}
	fclose( fp );
}

static void
loadpal( fp )
FILE	*fp;
{
	fread( palette, sizeof( unsigned short ), 16, fp );
	SetPalet();
}

static void
mkvpat( src )
unsigned short	*src;
{
	int	i, k, l;
	unsigned short	*rptr= src;
	unsigned char	*wptr= pat,
					*w2= vpat;

	for( i= 0; i < 256; ){
		for( k= 0; k < 4; k++ ){
			for( l= 0; l < 8; l++ ){
				*wptr++= *rptr;
				*w2++= *rptr++;
			}
			w2+= 128 - 8;
		}
		if( ++i & 15 ){
			w2-= 128 * 4 - 8;
		}else {
			w2-= 128 - 8;
		}
	}
}

static void
mkpat( src, w, h )
unsigned short	*src;
int	w;
int	h;
{
	int	i, j, k, l, m;
	unsigned short	*head, *rptr;
	unsigned char	*wptr, *w2;

	m= 0;
	wptr= pat;
	w2= vpat;
	head= src;
	for( i= 0; i < h; i++ ){
		for( j= 0; j < w / 8; j++ ){
			rptr= head;
			for( k= 0; k < 4; k++ ){
				for( l= 0; l < 8; l++ ){
					*wptr++= *rptr;
					*w2++= *rptr++;
				}
				rptr+= w - 8;
				w2+= 128 - 8;
			}
			head+= 8;
			if( ++m == 16 ){
				m= 0;
				w2-= 128 - 8;
			}else {
				w2-= 128 * 4 - 8;
			}
		}
		head+= w * 3;
	}
}

static void
loadpat( fp )
FILE	*fp;
{
	int	tmp;
	int	width, height, size;
	unsigned char	buf[4];
	unsigned short	*pat;

	tmp= fread( buf, sizeof( unsigned char ), 4, fp );
	if( tmp < 4 ){
		ConsolePrint( "is not pat file\n" );
		return;
	}
	width= ( buf[0] << 8 ) + buf[1];
	height= ( buf[2] << 8 ) + buf[3];
	if( width < 8 || width % 8 || height < 4 || height % 4 ){
		ConsolePrint( "cannot devide\n" );
		return;
	}
	size= width * height;
	if( size > 32 * 256 ){
		size= 32 * 256;
		height= size / width;
	}
	pat= malloc( sizeof( unsigned short) * size );
	if( !pat ){
		ConsolePrint( "memory exhausted\n" );
		return;
	}

	tmp= fread( pat, sizeof( unsigned short ), size, fp );
	if( tmp != size ){
		ConsolePrint( "pat file broken\n" );
		free( pat );
		return;
	}

	mkpat( pat, width, height / 4 );
	free( pat );
}

static void
loadmap( fp )
FILE	*fp;
{
	int	i, j;
	unsigned char	buf[32], *rptr;
	unsigned char	*wptr;

	loadpal( fp );

	rptr= malloc( sizeof( unsigned short ) * 32 * 256 );
	if( !rptr ){
		ConsolePrint( "memory exhausted\n" );
		return;
	}
	fread( rptr, sizeof( unsigned short ), 32 * 256, fp );
	mkvpat( rptr );
	free( rptr );

	wptr= map + 24 * ( 1024 - 1 );
	for( i= 0; i < 1024; i++ ){
		fread( buf, sizeof( unsigned char ), 32, fp );
		rptr= buf + 8;
		for( j= 0; j < 24; j++ )	*wptr++= *rptr++;
		wptr-= 48;
	}
}

void
loadfile( fname )
unsigned char	*fname;
{
	FILE	*fp;
	unsigned char	*ptr;
	int	c= 0;

	ConsoleOpen();
	if( !fname || !*fname )	return;

	ptr= strrchr( fname, '.' );
	if( !ptr )	return;

	if( !strcmpi( ++ptr, "pat" ) ){
		c= 1;
	}else if( !strcmpi( ptr, "pal" ) ){
		c= 2;
	}else if( !strcmpi( ptr, "map" ) ){
		c= 3;
	}
	if( !c )	return;

	ConsolePrint( fname );
	ConsolePrint( "\n" );

	fp= fopen( fname, "rb" );
	if( !fp ){
		ConsolePrint( "file cannot open\n" );
		return;
	}

	switch( c ){
	case 1:
		loadpat( fp );
		break;
	case 2:
		loadpal( fp );
		break;
	case 3:
		loadmap( fp );
		break;
	}
	fclose( fp );
}
